(function (exports) {
  'use strict';

  var utils = require('./utils');

  /**
   * Computes a hash of an 'obj'.
   * Hash of a:
   *  string is string
   *  number is number as string
   *  object is either result of calling $$hashKey function on the object or uniquely generated id,
   *         that is also assigned to the $$hashKey property of the object.
   *
   * @param obj
   * @returns {string} hash string such that the same input will have the same hash string.
   *         The resulting string key is in 'type:hashKey' format.
   */
  var hashKey = function hashKey(obj) {
    var objType = typeof obj,
      key;

    if (objType === 'object' && obj !== null) {
      if (typeof (key = obj.$$hashKey) === 'function') {
        // must invoke on object to keep the right this
        key = obj.$$hashKey();
      } else if (key === undefined) {
        key = obj.$$hashKey = utils.nextUid();
      }
    } else {
      key = obj;
    }

    return objType + ':' + key;
  };

  /**
   * HashMap which can use objects as keys
   */
  exports.HashMap = function HashMap(array) {
    utils.forEach(array, this.put, this);
  };

  exports.HashMap.prototype = {
    /**
     * Store key value pair
     * @param key key to store can be any type
     * @param value value to store can be any type
     */
    put: function (key, value) {
      this[hashKey(key)] = value;
    },

    /**
     * @param key
     * @returns the value for the key
     */
    get: function (key) {
      return this[hashKey(key)];
    },

    /**
     * Remove the key/value pair
     * @param key
     */
    remove: function (key) {
      var value = this[key = hashKey(key)];
      delete this[key];
      return value;
    }
  };

})(typeof exports === 'undefined'? require('./hashmap') : exports);
